<?php

namespace Drupal\restrict_ip\EventSubscriber;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\DependencyInjection\ContainerInjectionInterface;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use Drupal\Core\Routing\UrlGeneratorInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\Core\Url;
use Drupal\restrict_ip\Service\RestrictIpServiceInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;
use Symfony\Component\HttpFoundation\RedirectResponse;
use Symfony\Component\HttpKernel\Event\RequestEvent;
use Symfony\Component\HttpKernel\KernelEvents;
use Drupal\Core\Messenger\Messenger;

/**
 * The event subscriber for ip restriction events.
 */
class RestrictIpEventSubscriber implements EventSubscriberInterface, ContainerInjectionInterface {
  use StringTranslationTrait;
  /**
   * The Restrict IP Service.
   *
   * @var \Drupal\restrict_ip\Service\RestrictIpServiceInterface
   */
  protected $restrictIpService;

  /**
   * The Logger Factory service.
   *
   * @var \Drupal\Core\Logger\LoggerChannelFactoryInterface
   */
  protected $loggerFactory;

  /**
   * The Module Handler service.
   *
   * @var \Drupal\Core\Extension\ModuleHandlerInterface
   */
  protected $moduleHandler;

  /**
   * The Url Generator service.
   *
   * @var \Drupal\Core\Routing\UrlGeneratorInterface
   */
  protected $urlGenerator;

  /**
   * The Restrict IP configuration settings.
   *
   * @var \Drupal\Core\Config\ImmutableConfig
   */
  protected $config;

  /**
   * The Drupal messenger service object.
   *
   * @var \Drupal\Core\Messenger\Messenger
   */
  protected $messenger;

  /**
   * Creates an instance of the RestrictIpEventSubscriber class.
   *
   * @param \Drupal\restrict_ip\Service\RestrictIpServiceInterface $restrictIpService
   *   The restrict IP service.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $configFactory
   *   The Config Factory service.
   * @param \Drupal\Core\Logger\LoggerChannelFactoryInterface $loggerFactory
   *   The Logger Factory service.
   * @param \Drupal\Core\Extension\ModuleHandlerInterface $moduleHandler
   *   The Module Handler service.
   * @param \Drupal\Core\Routing\UrlGeneratorInterface $urlGenerator
   *   The Url Generator service.
   * @param \Drupal\Core\Messenger\Messenger $messenger
   *   The messenger service object.
   */
  public function __construct(RestrictIpServiceInterface $restrictIpService, ConfigFactoryInterface $configFactory, LoggerChannelFactoryInterface $loggerFactory, ModuleHandlerInterface $moduleHandler, UrlGeneratorInterface $urlGenerator, Messenger $messenger) {
    $this->restrictIpService = $restrictIpService;
    $this->loggerFactory = $loggerFactory;
    $this->moduleHandler = $moduleHandler;
    $this->urlGenerator = $urlGenerator;
    $this->config = $configFactory->get('restrict_ip.settings');
    $this->messenger = $messenger;
  }

  /**
   * {@inheritDoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('restrict_ip.service'),
      $container->get('config.factory'),
      $container->get('logger.factory'),
      $container->get('module_handler'),
      $container->get('url_generator'),
      $container->get('messenger'),
    );
  }

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents() {
    // On page load, we need to check for whether the user should be blocked by
    // IP:
    $events[KernelEvents::REQUEST][] = ['checkIpRestricted'];

    return $events;
  }

  /**
   * Check if response needs to be restricted.
   */
  public function checkIpRestricted(RequestEvent $event) {
    unset($_SESSION['restrict_ip']);

    $this->restrictIpService->testForBlock();

    if ($this->restrictIpService->userIsBlocked()) {
      if ($this->restrictIpService->getCurrentPath() != '/restrict_ip/access_denied') {
        if ($this->moduleHandler->moduleExists('dblog') && $this->config->get('dblog')) {
          $this->loggerFactory->get('Restrict IP')->warning('Access to the path %path was blocked for the IP address %ip_address', [
            '%path' => $this->restrictIpService->getCurrentPath(),
            '%ip_address' => $this->restrictIpService->getCurrentUserIp(),
          ]);
        }

        if ($this->config->get('allow_role_bypass') && $this->config->get('bypass_action') === 'redirect_login_page') {
          // Redirect the user to the change password page.
          $url = Url::fromRoute('user.login');

          $event->setResponse(new RedirectResponse($url->toString()));
        }
        elseif (in_array($this->config->get('white_black_list'), [0, 1])) {
          $url = Url::fromRoute('restrict_ip.access_denied_page');

          $event->setResponse(new RedirectResponse($url->toString()));
        }
        else {
          $this->setMessage($this->t('The page you are trying to access cannot be accessed from your IP address.'));
          $url = $this->urlGenerator->generateFromRoute('<front>');
          $event->setResponse(new RedirectResponse($url));
        }
      }
    }
  }

  /**
   * Helper function to create a status message.
   */
  private function setMessage($message) {
    $this->messenger->addStatus($message);
  }

}
